Trait ImgExtMut

trait ImgExtMut<Pixel>

Additional methods that depend on buffer size

To use these methods you need:

use imgref::*;

Required Methods

fn rows_padded_mut(self: &mut Self) -> ChunksMut<'_, Pixel>

Iterate over the entire buffer as rows, including all padding

Rows will have up to stride width, but the last row may be shorter.

fn as_mut(self: &mut Self) -> ImgRefMut<'_, Pixel>

Borrow the container mutably

Implementors