Trait ImgExt
trait ImgExt<Pixel>
Additional methods that depend on buffer size
To use these methods you need:
use *;
Required Methods
fn width_padded(self: &Self) -> usizeMaximum possible width of the data, including the stride.
Panics
This method may panic if the underlying buffer is not at least
height()*stride()pixels large.fn height_padded(self: &Self) -> usizeHeight in number of full strides. If the underlying buffer is not an even multiple of strides, the last row is ignored.
Panics
This method may panic if the underlying buffer is not at least
height()*stride()pixels large.fn rows_padded(self: &Self) -> Chunks<'_, Pixel>Iterate over the entire buffer as rows, including all padding
Rows will have up to
stridewidth, but the last row may be shorter.fn as_ref(self: &Self) -> ImgRef<'_, Pixel>Borrow the container
Implementors
impl<Pixel, Container> ImgExt for Img<Container>