pub trait ImgExtMut<Pixel> {
// Required methods
fn rows_padded_mut(&mut self) -> ChunksMut<'_, Pixel>;
fn as_mut(&mut self) -> ImgRefMut<'_, Pixel>;
}Expand description
Additional methods that depend on buffer size
To use these methods you need:
use imgref::*;Required Methods
Sourcefn rows_padded_mut(&mut self) -> ChunksMut<'_, Pixel>
fn rows_padded_mut(&mut self) -> ChunksMut<'_, Pixel>
Iterate over the entire buffer as rows, including all padding
Rows will have up to stride width, but the last row may be shorter.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".